@@ -1,5 +1,5 @@
 
-#define _GNU_SOURCE             /* struct mmsghdr */
+#define _GNU_SOURCE /* struct mmsghdr */
 
 #include <sys/socket.h>
 
@@ -11,9 +11,10 @@
    the from addresses in [v_srcs], if supplied.  Use [hdrs], in
    particular, fully initializing and saving results there, including
    lengths of data read, flags, etc. */
-int recvmmsg_assume_fd_is_nonblocking(
-  value v_fd, struct iovec *iovecs, unsigned count, value v_srcs, struct mmsghdr *hdrs);
+int recvmmsg_assume_fd_is_nonblocking(value v_fd, struct iovec *iovecs,
+                                      unsigned count, value v_srcs,
+                                      struct mmsghdr *hdrs);
 
 #define RECVMMSG_MAX_COUNT 64
 
-#endif  /* JSC_RECVMMSG */
+#endif /* JSC_RECVMMSG */
