@@ -9,8 +9,7 @@ let run sub_cmd =
 
 let%expect_test "do nothing" =
   let%map () = run "noop" in
-  [%expect {|
-    DUMMY_VARIABLE=DUMMY |}]
+  [%expect {| DUMMY_VARIABLE=DUMMY |}]
 ;;
 
 let%expect_test "clear" =
@@ -22,5 +21,6 @@ let%expect_test "set" =
   let%map () = run "set" in
   [%expect {|
     TEST_VAR=TEST_VALUE
-    DUMMY_VARIABLE=DUMMY |}]
+    DUMMY_VARIABLE=DUMMY
+    |}]
 ;;
