@@ -6,13 +6,14 @@ let%expect_test "[File_descr.sexp_of_t]" =
   print_s
     [%sexp
       (List.map [ -1; 0; 1; 2; 3 ] ~f:(fun i -> i, i |> File_descr.of_int)
-       : (int * File_descr.t) list)];
+        : (int * File_descr.t) list)];
   [%expect {|
     ((-1 -1)
      (0  0)
      (1  1)
      (2  2)
-     (3  _)) |}]
+     (3  _))
+    |}]
 ;;
 
 let%test_unit "[Error]" =
@@ -97,26 +98,26 @@ let%test_unit "fork_exec ~env last binding takes precedence" =
     ~finally:remove
     (Filename_unix.temp_file "test" "fork_exec.env.last-wins")
     ~f:(fun temp_file ->
-      let var_in_child env =
-        waitpid_exn
-          (fork_exec
-             ()
-             ~env
-             ~prog:"sh"
-             ~argv:[ "sh"; "-c"; "echo -n ${VAR-undefined} > " ^ temp_file ]);
-        In_channel.read_all temp_file
-      in
-      let env = [ "VAR", "first"; "VAR", "last" ] in
-      List.iter
-        [ `Replace_raw (List.map env ~f:(fun (v, s) -> v ^ "=" ^ s))
-        ; `Replace env
-        ; `Extend env
-        ; `Override (List.map env ~f:(fun (v, s) -> v, Some s))
-        ]
-        ~f:(fun env -> [%test_result: string] ~expect:"last" (var_in_child env));
-      Unix.putenv ~key:"VAR" ~data:"in-process";
-      let env = `Override [ "VAR", None ] in
-      [%test_result: string] ~expect:"undefined" (var_in_child env))
+    let var_in_child env =
+      waitpid_exn
+        (fork_exec
+           ()
+           ~env
+           ~prog:"sh"
+           ~argv:[ "sh"; "-c"; "echo -n ${VAR-undefined} > " ^ temp_file ]);
+      In_channel.read_all temp_file
+    in
+    let env = [ "VAR", "first"; "VAR", "last" ] in
+    List.iter
+      [ `Replace_raw (List.map env ~f:(fun (v, s) -> v ^ "=" ^ s))
+      ; `Replace env
+      ; `Extend env
+      ; `Override (List.map env ~f:(fun (v, s) -> v, Some s))
+      ]
+      ~f:(fun env -> [%test_result: string] ~expect:"last" (var_in_child env));
+    Unix.putenv ~key:"VAR" ~data:"in-process";
+    let env = `Override [ "VAR", None ] in
+    [%test_result: string] ~expect:"undefined" (var_in_child env))
 ;;
 
 let%test_module _ =
@@ -146,6 +147,17 @@ let%test_module _ =
   end)
 ;;
 
+let%expect_test "close-on-exec" =
+  let r, w = pipe ~close_on_exec:true () in
+  clear_close_on_exec r;
+  require_equal [%here] (module Bool) false (get_close_on_exec r);
+  [%expect {| |}];
+  require_equal [%here] (module Bool) true (get_close_on_exec w);
+  [%expect {| |}];
+  close r;
+  close w
+;;
+
 let%test_unit _ =
   let test = "unix_test_file" in
   let rm_test () =
@@ -192,7 +204,7 @@ module Unix_tm_for_testing = struct
     ; tm_yday : int
     ; tm_isdst : bool
     }
-  [@@deriving fields, sexp_of]
+  [@@deriving fields ~iterators:for_all, sexp_of]
 
   let equal t1 t2 =
     let eq_int f = Field.get f t1 = Field.get f t2 in
@@ -243,7 +255,7 @@ let%expect_test "strptime trailing input" =
   print_s
     [%sexp
       (strptime ~allow_trailing_input:true ~fmt:"%Y-%m-%d" "2012-05-23 10:14:23"
-       : Unix_tm_for_testing.t)];
+        : Unix_tm_for_testing.t)];
   [%expect
     {|
     ((tm_sec   0)
@@ -254,7 +266,8 @@ let%expect_test "strptime trailing input" =
      (tm_year  112)
      (tm_wday  3)
      (tm_yday  143)
-     (tm_isdst false)) |}];
+     (tm_isdst false))
+    |}];
   require_does_raise [%here] (fun () -> strptime ~fmt:"%Y-%m-%d" "2012-05-23 10:14:23");
   [%expect {| (Failure "unix_strptime: did not consume entire input") |}]
 ;;
@@ -399,7 +412,8 @@ let%expect_test "[symlink] arguments are correct" =
     {|
     (readlink (target_of_link_name target))
     (raised (
-      Unix.Unix_error "No such file or directory" readlink "((filename target))")) |}]
+      Unix.Unix_error "No such file or directory" readlink "((filename target))"))
+    |}]
 ;;
 
 let%test_module "the search path passed to [create_process_env] has an effect" =
@@ -418,6 +432,46 @@ let%test_module "the search path passed to [create_process_env] has an effect" =
   end)
 ;;
 
+let%test_unit "create_process_with_fds works" =
+  let str = "hello" in
+  let len = String.length str in
+  let test ~prog ~args ~stdin ~stdout =
+    let result =
+      Unix.create_process_with_fds ~prog ~args ~stdin ~stdout ~stderr:Generate ()
+    in
+    waitpid_exn result.pid;
+    result
+  in
+  let check_output fd =
+    let buf = Bytes.create len in
+    let bytes_read = read fd ~buf in
+    [%test_result: int] bytes_read ~expect:len;
+    [%test_result: string] (Bytes.to_string buf) ~expect:str
+  in
+  (* Test stdin *)
+  let stdin_r, stdin_w = pipe ~close_on_exec:true () in
+  let bytes_written = write stdin_w ~buf:(Bytes.of_string str) in
+  [%test_result: int] bytes_written ~expect:len;
+  close stdin_w;
+  let result =
+    test ~prog:"/bin/cat" ~args:[] ~stdin:(Use_this stdin_r) ~stdout:Generate
+  in
+  check_output result.stdout;
+  (* Test stdout *)
+  let test_file = "create_process_with_fds_test_file" in
+  let rm_test_file () =
+    try unlink test_file with
+    | _ -> ()
+  in
+  rm_test_file ();
+  let stdout = openfile test_file ~mode:[ O_CREAT; O_WRONLY ] in
+  let _ =
+    test ~prog:"/bin/echo" ~args:[ str ] ~stdin:Generate ~stdout:(Use_this stdout)
+  in
+  with_file test_file ~mode:[ O_RDONLY ] ~f:check_output;
+  rm_test_file ()
+;;
+
 let%expect_test "unix socket path limit workaround" =
   if Sys_unix.file_exists_exn "/proc"
   then
@@ -439,8 +493,7 @@ let%expect_test ("Clock.get_cpuclock_for" [@tags "64-bits-only"]) =
   (* This pid is too large to be real  *)
   let bad_pid = Pid.of_int 100_000_000 in
   require_does_raise [%here] (fun () -> get_cpuclock_for bad_pid);
-  [%expect {|
-    (Unix.Unix_error "No such process" clock_getcpuclockid "") |}];
+  [%expect {| (Unix.Unix_error "No such process" clock_getcpuclockid "") |}];
   let clock = get_cpuclock_for (Unix.getpid ()) in
   let cputime = gettime (Custom clock) in
   (* Testing cpu clocks are hard, but this doesn't crash, and we've definitely accrued cpu
@@ -508,8 +561,8 @@ let%test_unit "readdir_detailed" =
     raise_s
       [%sexp
         "couldn't find file in directory"
-      , `dir_contents (l : Readdir_detailed.t list)
-      , `basename (cwd_base : string)]
+        , `dir_contents (l : Readdir_detailed.t list)
+        , `basename (cwd_base : string)]
   | Some x ->
     (match x.kind with
      | None | Some S_DIR -> ()
