@@ -15,11 +15,11 @@
 #endif
 
 #ifdef __GLIBC__
-CAMLprim value core_unix_realpath(value v_path)
-{
+CAMLprim value core_unix_realpath(value v_path) {
   const char *path = String_val(v_path);
   char *res = realpath(path, NULL);
-  if (res == NULL) uerror("realpath", v_path);
+  if (res == NULL)
+    uerror("realpath", v_path);
   else {
     value v_res = caml_copy_string(res);
     free(res);
@@ -27,14 +27,14 @@ CAMLprim value core_unix_realpath(value v_path)
   }
 }
 #else
-CAMLprim value core_unix_realpath(value v_path)
-{
+CAMLprim value core_unix_realpath(value v_path) {
   char *path = String_val(v_path);
   /* [realpath] is inherently broken without GNU-extension, and this
      seems like a reasonable thing to do if we do not build against
      GLIBC. */
   char resolved_path[JANE_PATH_MAX];
-  if (realpath(path, resolved_path) == NULL) uerror("realpath", v_path);
+  if (realpath(path, resolved_path) == NULL)
+    uerror("realpath", v_path);
   return caml_copy_string(resolved_path);
 }
 #endif
