@@ -10,7 +10,8 @@ let%expect_test "list_intersect" =
     ((4  4)
      (5  7)
      (9  10)
-     (14 15)) |}]
+     (14 15))
+    |}]
 ;;
 
 let make_stable_unit_tests_v1 ~coerce ~non_empty =
@@ -35,54 +36,74 @@ let%test_module _ =
 
     let%test_module "Interval.V1.Float" =
       (module Stable_unit_test.Make (struct
-           include V1.Float
+        include V1.Float
 
-           let equal = [%compare.equal: t]
+        let equal = [%compare.equal: t]
 
-           let tests =
-             make_stable_unit_tests_v1
-               ~coerce:V1.Private.to_float
-               ~non_empty:
-                 [ ( (1.5, 120.)
-                   , "(1.5 120)"
-                   , "\000\000\000\000\000\000\000\248?\000\000\000\000\000\000^@" )
-                 ]
-           ;;
-         end))
+        let tests =
+          make_stable_unit_tests_v1
+            ~coerce:V1.Private.to_float
+            ~non_empty:
+              [ ( (1.5, 120.)
+                , "(1.5 120)"
+                , "\000\000\000\000\000\000\000\248?\000\000\000\000\000\000^@" )
+              ]
+        ;;
+      end))
+    ;;
+
+    let%expect_test "Interval.V1.Float.t_sexp_grammar" =
+      print_endline [%sexp_digest: V1.Float.t];
+      [%expect {| 6af0cb32d43233f53d0fe7d78544fe29 |}]
     ;;
 
     let%test_module "Interval.V1.Int" =
       (module Stable_unit_test.Make (struct
-           include V1.Int
+        include V1.Int
 
-           let equal = [%compare.equal: t]
+        let equal = [%compare.equal: t]
 
-           let tests =
-             make_stable_unit_tests_v1
-               ~coerce:V1.Private.to_int
-               ~non_empty:[ (-5, 789), "(-5 789)", "\000\255\251\254\021\003" ]
-           ;;
-         end))
+        let tests =
+          make_stable_unit_tests_v1
+            ~coerce:V1.Private.to_int
+            ~non_empty:[ (-5, 789), "(-5 789)", "\000\255\251\254\021\003" ]
+        ;;
+      end))
+    ;;
+
+    let%expect_test "Interval.V1.Int.t_sexp_grammar" =
+      print_endline [%sexp_digest: V1.Int.t];
+      [%expect {| cc2bf35f949d1c1381da1c176ca3cce7 |}]
     ;;
 
     let%test_module "Interval.V1.Ofday" =
       (module Stable_unit_test.Make (struct
-           include V1.Ofday
+        include V1.Ofday
 
-           let equal = [%compare.equal: t]
+        let equal = [%compare.equal: t]
 
-           let tests =
-             let t1 = Time_float.Ofday.create ~hr:7 ~min:30 ~sec:7 ~ms:12 ~us:5 () in
-             let t2 = Time_float.Ofday.create ~hr:9 ~min:45 ~sec:8 ~ms:0 ~us:1 () in
-             make_stable_unit_tests_v1
-               ~coerce:V1.Private.to_ofday
-               ~non_empty:
-                 [ ( (t1, t2)
-                   , "(07:30:07.012005 09:45:08.000001)"
-                   , "\000\153\158\176\196\192_\218@\223\024\002\000\128$\225@" )
-                 ]
-           ;;
-         end))
+        let tests =
+          let t1 = Time_float.Ofday.create ~hr:7 ~min:30 ~sec:7 ~ms:12 ~us:5 () in
+          let t2 = Time_float.Ofday.create ~hr:9 ~min:45 ~sec:8 ~ms:0 ~us:1 () in
+          make_stable_unit_tests_v1
+            ~coerce:V1.Private.to_ofday
+            ~non_empty:
+              [ ( (t1, t2)
+                , "(07:30:07.012005 09:45:08.000001)"
+                , "\000\153\158\176\196\192_\218@\223\024\002\000\128$\225@" )
+              ]
+        ;;
+      end))
+    ;;
+
+    let%expect_test "Interval.V1.Ofday.t_sexp_grammar" =
+      print_endline [%sexp_digest: V1.Ofday.t];
+      [%expect {| 1893c2cb5f9a4f741e0751d13b2d812e |}]
+    ;;
+
+    let%expect_test "Interval.V1.Ofday_ns.t_sexp_grammar" =
+      print_endline [%sexp_digest: V1.Ofday_ns.t];
+      [%expect {| 62c867fbc256b26e42454ebee3d48b09 |}]
     ;;
   end)
 ;;
@@ -249,7 +270,7 @@ let%test_module "vs array" =
         interval_and_nearby_int
         ~sexp_of:[%sexp_of: t * int]
         ~f:(fun (t, i) ->
-          [%test_result: bool] ~expect:(Array.mem ~equal:Int.equal (to_array t) i) (mem t i))
+        [%test_result: bool] ~expect:(Array.mem ~equal:Int.equal (to_array t) i) (mem t i))
     ;;
 
     let%test_unit "binary_search" =
@@ -296,8 +317,7 @@ let%test_module _ =
 
     let%expect_test _ =
       print_s [%sexp (Int.convex_hull intervals : Int.t)];
-      [%expect {|
-        (2 7) |}]
+      [%expect {| (2 7) |}]
     ;;
 
     let intervals =
@@ -306,11 +326,118 @@ let%test_module _ =
 
     let%expect_test _ =
       print_s [%sexp (Int.convex_hull intervals : Int.t)];
-      [%expect {|
-        (2 6) |}]
+      [%expect {| (2 6) |}]
     ;;
 
     let intervals = [ Int.empty; Int.empty ]
     let%test _ = Int.is_empty (Int.convex_hull intervals)
   end)
 ;;
+
+(* Tests for set functions *)
+let%test_module _ =
+  (module struct
+    module Set = Interval.Set
+
+    type t = int Set.t [@@deriving sexp]
+
+    (** Turn an unsorted list of bounds into a sequence. First dedup, and then pair things
+        up nicely.  If there's an odd number of elements, drop the last one. *)
+    let make_intervals bounds =
+      let[@tail_mod_cons] rec pair_up list =
+        match list with
+        | [] -> []
+        | [ _ ] -> []
+        | hd1 :: hd2 :: tl -> (hd1, hd2) :: pair_up tl
+      in
+      List.dedup_and_sort bounds ~compare:Int.compare |> pair_up
+    ;;
+
+    let%expect_test "make_intervals" =
+      let test l = print_s [%sexp (make_intervals l : (int * int) list)] in
+      test [];
+      [%expect {| () |}];
+      test [ 1; 3; 4 ];
+      [%expect {| ((1 3)) |}];
+      test [ 5; 1; 10; 11; 1; 7; 9; 2; 5 ];
+      [%expect {|
+        ((1 2)
+         (5 7)
+         (9 10))
+        |}]
+    ;;
+
+    let quickcheck_generator =
+      Quickcheck.Generator.map [%quickcheck.generator: int list] ~f:(fun bounds ->
+        Set.create_exn (make_intervals bounds))
+    ;;
+
+    let%expect_test "Generated intervals sets are very likely both to contain and not \
+                     contain random ints"
+      =
+      let gen = [%quickcheck.generator: t * int] in
+      Quickcheck.test_can_generate gen ~trials:10 ~f:(fun (t, x) -> Set.contains t x);
+      Quickcheck.test_can_generate gen ~trials:10 ~f:(fun (t, x) ->
+        not (Set.contains t x))
+    ;;
+
+    (* As long as the tests are passing anyway, there's no reason to bother writing a
+       sophisticated shrinker. *)
+    let quickcheck_shrinker = Base_quickcheck.Shrinker.atomic
+
+    let%quick_test "union" =
+      fun (t1 : t) (t2 : t) (xs : int list) ->
+      let t3 = Set.union t1 t2 in
+      List.iter xs ~f:(fun x ->
+        [%test_result: bool]
+          (Set.contains t3 x)
+          ~expect:(Set.contains t1 x || Set.contains t2 x))
+    ;;
+
+    let%quick_test "intersect" =
+      fun (t1 : t) (t2 : t) (xs : int list) ->
+      let t3 = Set.inter t1 t2 in
+      List.iter xs ~f:(fun x ->
+        [%test_result: bool]
+          (Set.contains t3 x)
+          ~expect:(Set.contains t1 x && Set.contains t2 x))
+    ;;
+
+    let%quick_test "union_list" =
+      fun (ts : t list) (xs : int list) ->
+      let t = Set.union_list ts in
+      List.iter xs ~f:(fun x ->
+        [%test_result: bool]
+          (Set.contains t x)
+          ~expect:(List.exists ts ~f:(fun t -> Set.contains t x)))
+    ;;
+
+    let t1 = Set.create_exn [ 1, 5; 6, 10; 15, 20 ]
+    let t2 = Set.create_exn [ 3, 12 ]
+
+    let%expect_test "union demo" =
+      print_s [%sexp (Set.union t1 t2 : int Set.t)];
+      [%expect {|
+        ((1  12)
+         (15 20))
+        |}]
+    ;;
+
+    let%expect_test "intersect demo" =
+      print_s [%sexp (Set.inter t1 t2 : int Set.t)];
+      [%expect {|
+        ((3 5)
+         (6 10))
+        |}]
+    ;;
+
+    let%expect_test "union_list demo" =
+      let ts = [ t1; t2; Set.create_exn [ 11, 13; 14, 15 ]; Set.create_exn [ 20, 22 ] ] in
+      print_s [%sexp (Set.union_list ts : int Set.t)];
+      [%expect {|
+        ((1  13)
+         (14 22))
+        |}]
+    ;;
+  end)
+;;
