@@ -14,16 +14,16 @@ let%test_unit _ =
   List.iter
     [ Iobuf.Expert.to_bigstring_shared; to_bigstring_shared_via_iovec ]
     ~f:(fun to_bstr ->
-      let iobuf = Iobuf.of_string "0123456789" in
-      let bstr0 = to_bstr iobuf in
-      [%test_result: Bigstring.t] bstr0 ~expect:(Bigstring.of_string "0123456789");
-      Iobuf.Poke.char iobuf ~pos:0 'X';
-      [%test_result: Bigstring.t] bstr0 ~expect:(Bigstring.of_string "X123456789");
-      let bstr1 = to_bstr iobuf ~pos:1 ~len:8 in
-      [%test_result: Bigstring.t] bstr1 ~expect:(Bigstring.of_string "12345678");
-      Iobuf.Poke.char iobuf ~pos:1 'X';
-      [%test_result: Bigstring.t] bstr1 ~expect:(Bigstring.of_string "X2345678");
-      [%test_result: Bigstring.t] bstr0 ~expect:(Bigstring.of_string "XX23456789"))
+    let iobuf = Iobuf.of_string "0123456789" in
+    let bstr0 = to_bstr iobuf in
+    [%test_result: Bigstring.t] bstr0 ~expect:(Bigstring.of_string "0123456789");
+    Iobuf.Poke.char iobuf ~pos:0 'X';
+    [%test_result: Bigstring.t] bstr0 ~expect:(Bigstring.of_string "X123456789");
+    let bstr1 = to_bstr iobuf ~pos:1 ~len:8 in
+    [%test_result: Bigstring.t] bstr1 ~expect:(Bigstring.of_string "12345678");
+    Iobuf.Poke.char iobuf ~pos:1 'X';
+    [%test_result: Bigstring.t] bstr1 ~expect:(Bigstring.of_string "X2345678");
+    [%test_result: Bigstring.t] bstr0 ~expect:(Bigstring.of_string "XX23456789"))
 ;;
 
 let%expect_test "fillf_float.Ok" =
@@ -50,19 +50,19 @@ type nonrec ok_or_eof = ok_or_eof =
 let iter_examples = Iobuf_test.Test_iobuf.iter_examples
 
 module Io_test (Ch : sig
-    type in_
+  type in_
 
-    val create_in : string -> in_
-    val close_in : in_ -> unit
-    val read : ([> write ], seek) Iobuf.t -> in_ -> ok_or_eof
+  val create_in : string -> in_
+  val close_in : in_ -> unit
+  val read : ([> write ], seek) Iobuf.t -> in_ -> ok_or_eof
 
-    type out_
+  type out_
 
-    val create_out : Unix.File_descr.t -> out_
-    val close_out : out_ -> unit
-    val write : ([> read ], seek) Iobuf.t -> out_ -> unit
-    val peek_write : ([> read ], _) Iobuf.t -> out_ -> int
-  end) =
+  val create_out : Unix.File_descr.t -> out_
+  val close_out : out_ -> unit
+  val write : ([> read ], seek) Iobuf.t -> out_ -> unit
+  val peek_write : ([> read ], _) Iobuf.t -> out_ -> int
+end) =
 struct
   let%test_unit "write + read" =
     iter_examples ~f:(fun t string ~pos ->
@@ -118,35 +118,35 @@ let output = output
 let input = input
 
 include Io_test (struct
-    type in_ = In_channel.t
+  type in_ = In_channel.t
 
-    let create_in file = In_channel.create file
-    let close_in = In_channel.close
+  let create_in file = In_channel.create file
+  let close_in = In_channel.close
 
-    type out_ = Out_channel.t
+  type out_ = Out_channel.t
 
-    let create_out = Unix.out_channel_of_descr
-    let close_out = Out_channel.close
-    let write = output
-    let peek_write = Peek.output
-    let read = input
-  end)
+  let create_out = Unix.out_channel_of_descr
+  let close_out = Out_channel.close
+  let write = output
+  let peek_write = Peek.output
+  let read = input
+end)
 
 let read = read
 let write = write
 
 include Io_test (struct
-    type in_ = Unix.File_descr.t
-    type out_ = in_
+  type in_ = Unix.File_descr.t
+  type out_ = in_
 
-    let create_in file = Unix.openfile ~mode:[ Unix.O_RDONLY ] file
-    let close_in fd = Unix.close fd
-    let create_out = Fn.id
-    let close_out = close_in
-    let read = read
-    let peek_write = Peek.write
-    let write = write
-  end)
+  let create_in file = Unix.openfile ~mode:[ Unix.O_RDONLY ] file
+  let close_in fd = Unix.close fd
+  let create_out = Fn.id
+  let close_out = close_in
+  let read = read
+  let peek_write = Peek.write
+  let write = write
+end)
 
 let read_assume_fd_is_nonblocking = read_assume_fd_is_nonblocking
 let write_assume_fd_is_nonblocking = write_assume_fd_is_nonblocking
@@ -229,15 +229,15 @@ let sendto_and_recvfrom recvfrom recv_fd sendto ~sendto_name =
       let sender =
         Thread.create
           (fun () ->
-             let send_fd = Unix.(socket ~domain:PF_INET ~kind:SOCK_DGRAM ~protocol:0 ()) in
-             iter_examples ~f:(fun t string ~pos:_ ->
-               Fill.stringo t string;
-               Iobuf.flip_lo t;
-               retry_until_ready (fun () ->
-                 Unix.Syscall_result.Unit.ok_or_unix_error_exn
-                   (sendto t send_fd send_addr)
-                   ~syscall_name:sendto_name);
-               [%test_pred: (_, _) Iobuf.Hexdump.t] (fun buf -> Iobuf.is_empty buf) t))
+            let send_fd = Unix.(socket ~domain:PF_INET ~kind:SOCK_DGRAM ~protocol:0 ()) in
+            iter_examples ~f:(fun t string ~pos:_ ->
+              Fill.stringo t string;
+              Iobuf.flip_lo t;
+              retry_until_ready (fun () ->
+                Unix.Syscall_result.Unit.ok_or_unix_error_exn
+                  (sendto t send_fd send_addr)
+                  ~syscall_name:sendto_name);
+              [%test_pred: (_, _) Iobuf.Hexdump.t] (fun buf -> Iobuf.is_empty buf) t))
           ~on_uncaught_exn:`Print_to_stderr
           ()
       in
@@ -363,7 +363,7 @@ let check_msgs ?(int_size = 2) ?(be = false) file =
     let len_before = length t in
     (match
        read_assume_fd_is_nonblocking t fd |> Unix.Syscall_result.Unit.to_result
-     (* doesn't allocate *)
+       (* doesn't allocate *)
      with
      | Error (EAGAIN | EINTR | EWOULDBLOCK) -> ()
      | Error e -> raise (Unix.Unix_error (e, "read", ""))
@@ -476,7 +476,8 @@ let%expect_test "[In_channel.fold_lines_raw]" =
   [%expect {|
     ((buf a) (i 0))
     ((buf bc) (i 1))
-    ((buf def) (i 2)) |}]
+    ((buf def) (i 2))
+    |}]
 ;;
 
 let correct_answer_for_benchmark =
