@@ -41,43 +41,6 @@ module Bound_to_interface = struct
   [@@deriving sexp_of]
 end
 
-(* We use [Int63] rather than [Int] because these flags use 32 bits. *)
-
-module Epoll_flags (Flag_values : sig
-    val in_ : Int63.t
-    val out : Int63.t
-
-    (* val rdhup   : Int63.t *)
-    val pri : Int63.t
-    val err : Int63.t
-    val hup : Int63.t
-    val et : Int63.t
-    val oneshot : Int63.t
-  end) =
-struct
-  let none = Int63.zero
-
-  include Flag_values
-
-  include Flags.Make (struct
-      let allow_intersecting = false
-      let should_print_error = true
-      let remove_zero_flags = false
-
-      let known =
-        [ in_, "in"
-        ; out, "out"
-        ; (* rdhup, "rdhup"; *)
-          pri, "pri"
-        ; err, "err"
-        ; hup, "hup"
-        ; et, "et"
-        ; oneshot, "oneshot"
-        ]
-      ;;
-    end)
-end
-
 module Priority : sig
   type t [@@deriving sexp]
 
@@ -249,41 +212,7 @@ module Null_toplevel = struct
   let settcpopt_string = u "Linux_ext.settcpopt_string"
   let peer_credentials = u "Linux_ext.peer_credentials"
 
-  module Epoll = struct
-    module Flags = Epoll_flags (struct
-        let in_ = Int63.of_int (1 lsl 0)
-        let out = Int63.of_int (1 lsl 1)
-
-        (* let rdhup   = Int63.of_int (1 lsl 2) *)
-        let pri = Int63.of_int (1 lsl 3)
-        let err = Int63.of_int (1 lsl 4)
-        let hup = Int63.of_int (1 lsl 5)
-        let et = Int63.of_int (1 lsl 6)
-        let oneshot = Int63.of_int (1 lsl 7)
-      end)
-
-    type t = [ `Epoll_is_not_implemented ] [@@deriving sexp_of]
-
-    let create = Or_error.unimplemented "Linux_ext.Epoll.create"
-    let close _ = assert false
-    let invariant _ = assert false
-    let find _ _ = assert false
-    let find_exn _ _ = assert false
-    let set _ _ _ = assert false
-    let remove _ _ = assert false
-    let iter _ ~f:_ = assert false
-    let fold _ ~init:_ ~f:_ = assert false
-    let wait _ ~timeout:_ = assert false
-    let wait_timeout_after _ _ = assert false
-    let iter_ready _ ~f:_ = assert false
-    let fold_ready _ ~init:_ ~f:_ = assert false
-
-    module Expert = struct
-      let clear_ready _ = assert false
-    end
-
-    (* let pwait _ ~timeout:_ _      = assert false *)
-  end
+  module Epoll = Epoll.Impl
 end
 
 module Null : Linux_ext_intf.S = struct
@@ -327,11 +256,11 @@ module Null : Linux_ext_intf.S = struct
       let semaphore = Int63.of_int 0o1
 
       include Flags.Make (struct
-          let allow_intersecting = true
-          let should_print_error = true
-          let remove_zero_flags = false
-          let known = [ nonblock, "nonblock"; cloexec, "cloexec"; semaphore, "semaphore" ]
-        end)
+        let allow_intersecting = true
+        let should_print_error = true
+        let remove_zero_flags = false
+        let known = [ nonblock, "nonblock"; cloexec, "cloexec"; semaphore, "semaphore" ]
+      end)
     end
 
     let create = Or_error.unimplemented "Linux_ext.Eventfd.create"
@@ -353,11 +282,11 @@ module Null : Linux_ext_intf.S = struct
       let cloexec = Int63.of_int 0o2000000
 
       include Flags.Make (struct
-          let allow_intersecting = false
-          let should_print_error = true
-          let remove_zero_flags = false
-          let known = List.rev [ nonblock, "nonblock"; cloexec, "cloexec" ]
-        end)
+        let allow_intersecting = false
+        let should_print_error = true
+        let remove_zero_flags = false
+        let known = List.rev [ nonblock, "nonblock"; cloexec, "cloexec" ]
+      end)
     end
 
     type t = File_descr.t [@@deriving compare, sexp_of]
@@ -382,6 +311,44 @@ module Null : Linux_ext_intf.S = struct
     end
   end
 
+  module Memfd = struct
+    module Flags = struct
+      (* As per include/uapi/linux/memfd.h *)
+
+      let i63 = Int63.of_int
+      let cloexec = i63 0x0001
+      let allow_sealing = i63 0x0002
+      let hugetlb = i63 0x0004
+      let noexec_seal = i63 0x0008
+      let exec = i63 0x0010
+      let hugetlb_flag_encode_shift = 26
+      let huge_2mb = i63 (21 lsl hugetlb_flag_encode_shift)
+      let huge_1gb = i63 (30 lsl hugetlb_flag_encode_shift)
+
+      include Flags.Make (struct
+        let allow_intersecting = true (* huge_* flags intersect *)
+        let should_print_error = true
+        let remove_zero_flags = false
+
+        let known =
+          [ cloexec, "cloexec"
+          ; allow_sealing, "allow_sealing"
+          ; hugetlb, "hugetlb"
+          ; noexec_seal, "noexec_seal"
+          ; exec, "exec"
+          ; huge_2mb, "huge_2mb"
+          ; huge_1gb, "huge_1gb"
+          ]
+        ;;
+      end)
+    end
+
+    type t = File_descr.t [@@deriving sexp_of]
+
+    let to_file_descr = Fn.id
+    let create = Or_error.unimplemented "Linux_ext.Memfd.create"
+  end
+
   module Extended_file_attributes = struct
     module Get_attr_result = struct
       type t =
@@ -490,11 +457,11 @@ module Timerfd = struct
     let cloexec = cloexec ()
 
     include Flags.Make (struct
-        let allow_intersecting = false
-        let should_print_error = true
-        let remove_zero_flags = false
-        let known = List.rev [ nonblock, "nonblock"; cloexec, "cloexec" ]
-      end)
+      let allow_intersecting = false
+      let should_print_error = true
+      let remove_zero_flags = false
+      let known = List.rev [ nonblock, "nonblock"; cloexec, "cloexec" ]
+    end)
   end
 
   type t = File_descr.t [@@deriving compare, sexp_of]
@@ -533,7 +500,7 @@ module Timerfd = struct
     -> interval:Int63.t
     -> Syscall_result.Unit.t
     = "core_linux_timerfd_settime"
-  [@@noalloc]
+    [@@noalloc]
 
   let timerfd_settime t ~absolute ~initial ~interval =
     (* We could accept [interval < 0] or [initial < 0 when absolute], but then the
@@ -544,7 +511,7 @@ module Timerfd = struct
       raise_s
         [%sexp
           "timerfd_settime got invalid parameters (initial < 0 or interval < 0)."
-        , { timerfd = (t : t); initial : Int63.t; interval : Int63.t }];
+          , { timerfd = (t : t); initial : Int63.t; interval : Int63.t }];
     unsafe_timerfd_settime t absolute ~initial ~interval
     |> Syscall_result.Unit.ok_or_unix_error_exn ~syscall_name:"timerfd_settime"
   ;;
@@ -642,6 +609,68 @@ end
 
 module Timerfd = Null.Timerfd
 
+[%%endif]
+[%%ifdef JSC_MEMFD]
+
+module Memfd = struct
+  module Flags = struct
+    external cloexec : unit -> Int63.t = "core_linux_memfd_MFD_CLOEXEC"
+    external allow_sealing : unit -> Int63.t = "core_linux_memfd_MFD_ALLOW_SEALING"
+    external hugetlb : unit -> Int63.t = "core_linux_memfd_MFD_HUGETLB"
+    external noexec_seal : unit -> Int63.t = "core_linux_memfd_MFD_NOEXEC_SEAL"
+    external exec : unit -> Int63.t = "core_linux_memfd_MFD_EXEC"
+    external huge_2mb : unit -> Int63.t = "core_linux_memfd_MFD_HUGE_2MB"
+    external huge_1gb : unit -> Int63.t = "core_linux_memfd_MFD_HUGE_1GB"
+
+    let cloexec = cloexec ()
+    let allow_sealing = allow_sealing ()
+    let hugetlb = hugetlb ()
+    let noexec_seal = noexec_seal ()
+    let exec = exec ()
+    let huge_2mb = huge_2mb ()
+    let huge_1gb = huge_1gb ()
+
+    include Flags.Make (struct
+      let allow_intersecting = true (* huge_* flags intersect *)
+      let should_print_error = true
+      let remove_zero_flags = false
+
+      let known =
+        [ cloexec, "cloexec"
+        ; allow_sealing, "allow_sealing"
+        ; hugetlb, "hugetlb"
+        ; noexec_seal, "noexec_seal"
+        ; exec, "exec"
+        ; huge_2mb, "huge_2mb"
+        ; huge_1gb, "huge_1gb"
+        ]
+      ;;
+    end)
+  end
+
+  type t = File_descr.t [@@deriving compare, sexp_of]
+
+  external create
+    :  flags:Flags.t
+    -> initial_size:int
+    -> name:string
+    -> t
+    = "core_linux_memfd_create"
+
+  let create =
+    let create ?(flags = Flags.empty) ?(initial_size = 0) name =
+      create ~flags ~initial_size ~name
+    in
+    Or_error.return create
+  ;;
+
+  let to_file_descr t = t
+end
+
+[%%else]
+
+module Memfd = Null.Memfd
+
 [%%endif]
 [%%ifdef JSC_LINUX_EXT]
 
@@ -659,11 +688,11 @@ module Eventfd = struct
     let known = [ cloexec, "cloexec"; nonblock, "nonblock"; semaphore, "semaphore" ]
 
     include Flags.Make (struct
-        let allow_intersecting = true
-        let should_print_error = true
-        let known = known
-        let remove_zero_flags = false
-      end)
+      let allow_intersecting = true
+      let should_print_error = true
+      let known = known
+      let remove_zero_flags = false
+    end)
   end
 
   type t = File_descr.t [@@deriving compare, sexp_of]
@@ -725,22 +754,22 @@ module Sysinfo = struct
   let sysinfo =
     Ok
       (fun () ->
-         let raw = raw_sysinfo () in
-         { uptime = Time_float.Span.of_int_sec raw.Raw_sysinfo.uptime
-         ; load1 = raw.Raw_sysinfo.load1
-         ; load5 = raw.Raw_sysinfo.load5
-         ; load15 = raw.Raw_sysinfo.load15
-         ; total_ram = raw.Raw_sysinfo.total_ram
-         ; free_ram = raw.Raw_sysinfo.free_ram
-         ; shared_ram = raw.Raw_sysinfo.shared_ram
-         ; buffer_ram = raw.Raw_sysinfo.buffer_ram
-         ; total_swap = raw.Raw_sysinfo.total_swap
-         ; free_swap = raw.Raw_sysinfo.free_swap
-         ; procs = raw.Raw_sysinfo.procs
-         ; totalhigh = raw.Raw_sysinfo.totalhigh
-         ; freehigh = raw.Raw_sysinfo.freehigh
-         ; mem_unit = raw.Raw_sysinfo.mem_unit
-         })
+        let raw = raw_sysinfo () in
+        { uptime = Time_float.Span.of_int_sec raw.Raw_sysinfo.uptime
+        ; load1 = raw.Raw_sysinfo.load1
+        ; load5 = raw.Raw_sysinfo.load5
+        ; load15 = raw.Raw_sysinfo.load15
+        ; total_ram = raw.Raw_sysinfo.total_ram
+        ; free_ram = raw.Raw_sysinfo.free_ram
+        ; shared_ram = raw.Raw_sysinfo.shared_ram
+        ; buffer_ram = raw.Raw_sysinfo.buffer_ram
+        ; total_swap = raw.Raw_sysinfo.total_swap
+        ; free_swap = raw.Raw_sysinfo.free_swap
+        ; procs = raw.Raw_sysinfo.procs
+        ; totalhigh = raw.Raw_sysinfo.totalhigh
+        ; freehigh = raw.Raw_sysinfo.freehigh
+        ; mem_unit = raw.Raw_sysinfo.mem_unit
+        })
   ;;
 end
 
@@ -885,7 +914,6 @@ external raw_sched_getaffinity : pid:int -> int list = "core_linux_sched_getaffi
 
 let sched_getaffinity ?pid () = raw_sched_getaffinity ~pid:(pid_to_int_or_zero pid)
 
-
 (* defined in core_unix_stubs.c *)
 external gettid : unit -> int = "core_unix_gettid"
 
@@ -959,324 +987,7 @@ let get_bind_to_interface fd =
   | name -> Bound_to_interface.Only name
 ;;
 
-module Epoll = struct
-  external flag_epollin : unit -> Int63.t = "core_linux_epoll_EPOLLIN_flag"
-  external flag_epollout : unit -> Int63.t = "core_linux_epoll_EPOLLOUT_flag"
-
-  (* external flag_epollrdhup   : unit -> Int63.t  = "core_linux_epoll_EPOLLRDHUP_flag" *)
-  external flag_epollpri : unit -> Int63.t = "core_linux_epoll_EPOLLPRI_flag"
-  external flag_epollerr : unit -> Int63.t = "core_linux_epoll_EPOLLERR_flag"
-  external flag_epollhup : unit -> Int63.t = "core_linux_epoll_EPOLLHUP_flag"
-  external flag_epollet : unit -> Int63.t = "core_linux_epoll_EPOLLET_flag"
-  external flag_epolloneshot : unit -> Int63.t = "core_linux_epoll_EPOLLONESHOT_flag"
-
-  module Flags = Epoll_flags (struct
-      let in_ = flag_epollin ()
-      let out = flag_epollout ()
-
-      (* let rdhup   = flag_epollrdhup () *)
-      let pri = flag_epollpri ()
-      let err = flag_epollerr ()
-      let hup = flag_epollhup ()
-      let et = flag_epollet ()
-      let oneshot = flag_epolloneshot ()
-    end)
-
-  external epoll_create : unit -> File_descr.t = "core_linux_epoll_create"
-
-  (* Some justification for the below interface: Unlike select() and poll(), epoll() fills
-     in an array of ready events, analogous to a read() call where you pass in a buffer to
-     be filled.
-
-     Since this is at the core of the I/O loop, we'd like to avoid reallocating that
-     buffer on every call to poll.  We're allocating the array on the ocaml side (as a
-     Bigstring), then iterating through it in-place, reducing allocation, copies, and any
-     intermediate lists.  For very high message rates and many fds this could be a very
-     beneficial. *)
-  type ready_events = Bigstring.t
-
-  external epoll_sizeof_epoll_event : unit -> int = "core_linux_epoll_sizeof_epoll_event"
-  [@@noalloc]
-
-  external epoll_offsetof_readyfd : unit -> int = "core_linux_epoll_offsetof_readyfd"
-  [@@noalloc]
-
-  external epoll_offsetof_readyflags
-    :  unit
-    -> int
-    = "core_linux_epoll_offsetof_readyflags"
-  [@@noalloc]
-
-  let sizeof_epoll_event = epoll_sizeof_epoll_event ()
-  let offsetof_readyfd = epoll_offsetof_readyfd ()
-  let offsetof_readyflags = epoll_offsetof_readyflags ()
-
-  external epoll_ctl_add
-    :  File_descr.t
-    -> File_descr.t
-    -> Flags.t
-    -> unit
-    = "core_linux_epoll_ctl_add"
-
-  external epoll_ctl_mod
-    :  File_descr.t
-    -> File_descr.t
-    -> Flags.t
-    -> unit
-    = "core_linux_epoll_ctl_mod"
-
-  external epoll_ctl_del
-    :  File_descr.t
-    -> File_descr.t
-    -> unit
-    = "core_linux_epoll_ctl_del"
-
-  module Table = Bounded_int_table
-
-  module T = struct
-    type 'a t =
-      { epollfd : File_descr.t
-      ; (* [flags_by_fd] has one entry for each file-descr in the epoll set, and stores
-           the epoll flags that the kernel's epoll set currently has for that
-           file-descr.  Keeping our own representation of the kernel data structure is
-           useful for debugging, since the information appears in a human-readable way
-           in [sexp_of_t]'s output.  It also allows us to hide the distinction between
-           [epoll_ctl_add] and [epoll_ctl_mod], since we know which to use based on
-           whether the file descriptor is already being watched. *)
-        flags_by_fd : (File_descr.t, Flags.t) Table.t
-      ; max_ready_events : int
-      ; (* [num_ready_events] holds the number of ready events in [ready_events], as
-           determined by the last call to [wait]. *)
-        mutable num_ready_events : int
-      ; ready_events : 'a
-      }
-    [@@deriving fields, sexp_of]
-  end
-
-  open T
-
-  let epoll_readyfd t i =
-    Bigstring.unsafe_get_int32_le t ~pos:((i * sizeof_epoll_event) + offsetof_readyfd)
-    |> File_descr.of_int
-  ;;
-
-  let epoll_readyflags t i =
-    Bigstring.unsafe_get_int32_le t ~pos:((i * sizeof_epoll_event) + offsetof_readyflags)
-    |> Flags.of_int
-  ;;
-
-  type in_use = ready_events T.t
-
-  module Pretty = struct
-    type ready_event =
-      { file_descr : File_descr.t
-      ; flags : Flags.t
-      }
-    [@@deriving sexp_of]
-
-    type ready_events = ready_event array [@@deriving sexp_of]
-    type t = ready_events T.t [@@deriving sexp_of]
-  end
-
-  let to_pretty t =
-    { t with
-      ready_events =
-        Array.init t.num_ready_events ~f:(fun i ->
-          { Pretty.file_descr = epoll_readyfd t.ready_events i
-          ; flags = epoll_readyflags t.ready_events i
-          })
-    }
-  ;;
-
-  let sexp_of_in_use t = Pretty.sexp_of_t (to_pretty t)
-
-  type t = [ `Closed | `In_use of in_use ] ref [@@deriving sexp_of]
-
-  let close t =
-    match !t with
-    | `Closed -> ()
-    | `In_use { epollfd; _ } ->
-      t := `Closed;
-      Unix.close epollfd
-  ;;
-
-  let invariant t : unit =
-    match !t with
-    | `Closed -> ()
-    | `In_use t ->
-      (try
-         let check f field = f (Field.get field t) in
-         Fields.iter
-           ~epollfd:ignore
-           ~flags_by_fd:(check (Table.invariant ignore ignore))
-           ~max_ready_events:
-             (check (fun max_ready_events -> assert (max_ready_events > 0)))
-           ~num_ready_events:(check (fun num_ready -> assert (num_ready >= 0)))
-           ~ready_events:ignore
-       with
-       | exn ->
-         failwiths
-           ~here:[%here]
-           "Epoll.invariant failed"
-           (exn, t)
-           [%sexp_of: exn * in_use])
-  ;;
-
-  let create ~num_file_descrs ~max_ready_events =
-    if max_ready_events < 0
-    then
-      failwiths
-        ~here:[%here]
-        "Epoll.create got nonpositive max_ready_events"
-        max_ready_events
-        [%sexp_of: int];
-    ref
-      (`In_use
-         { epollfd = epoll_create ()
-         ; flags_by_fd =
-             Table.create
-               ~num_keys:num_file_descrs
-               ~key_to_int:File_descr.to_int
-               ~sexp_of_key:File_descr.sexp_of_t
-               ()
-         ; max_ready_events
-         ; num_ready_events = 0
-         ; ready_events = Bigstring.create (sizeof_epoll_event * max_ready_events)
-         })
-  ;;
-
-  let in_use_exn t =
-    match !t with
-    | `Closed -> failwith "attempt to use closed epoll set"
-    | `In_use r -> r
-  ;;
-
-  let find t file_descr =
-    let t = in_use_exn t in
-    Table.find t.flags_by_fd file_descr
-  ;;
-
-  let find_exn t file_descr =
-    let t = in_use_exn t in
-    Table.find_exn t.flags_by_fd file_descr
-  ;;
-
-  let iter t ~f =
-    let t = in_use_exn t in
-    Table.iteri t.flags_by_fd ~f:(fun ~key:file_descr ~data:flags -> f file_descr flags)
-  ;;
-
-  let fold t ~init ~f =
-    let t = in_use_exn t in
-    Table.fold t.flags_by_fd ~init ~f:(fun ~key ~data -> f key data)
-  ;;
-
-  let set t fd flags =
-    let t = in_use_exn t in
-    let already_present = Table.mem t.flags_by_fd fd in
-    (* Both [epoll_ctl_add] and [epoll_ctl_mod] may raise if the file descriptor does not
-       support polling. Perform these operations first and let them raise before modifying
-       the table to reflect the change in epoll state. *)
-    let () =
-      if already_present
-      then epoll_ctl_mod t.epollfd fd flags
-      else epoll_ctl_add t.epollfd fd flags
-    in
-    Table.set t.flags_by_fd ~key:fd ~data:flags
-  ;;
-
-  let remove t fd =
-    let t = in_use_exn t in
-    if Table.mem t.flags_by_fd fd then epoll_ctl_del t.epollfd fd;
-    Table.remove t.flags_by_fd fd
-  ;;
-
-  external epoll_wait
-    :  File_descr.t
-    -> ready_events
-    -> int
-    -> int
-    = "core_linux_epoll_wait"
-
-  let wait_internal t ~timeout_ms =
-    let t = in_use_exn t in
-    (* We clear [num_ready_events] because [epoll_wait] will invalidate [ready_events],
-       and we don't want another thread to observe [t] and see junk. *)
-    t.num_ready_events <- 0;
-    t.num_ready_events <- epoll_wait t.epollfd t.ready_events timeout_ms;
-    if t.num_ready_events = 0 then `Timeout else `Ok
-  ;;
-
-  let wait_timeout_after t span =
-    let timeout_ms =
-      if Time_ns.Span.( <= ) span Time_ns.Span.zero
-      then 0
-      else (
-        (* For positive timeouts, we use a minimum timeout of one millisecond, to ensure
-           that we are guaranteed that the timeout has passed when we wake up.  If we
-           allowed a positive sub-millisecond timeout, we would round down and end up
-           using a timeout of zero, causing [wait_internal] to return immediately.  Such
-           behaviour has been seen to cause Async to spin, repeatedly requesting slightly
-           smaller timeouts. *)
-        let span = Time_ns.Span.max span Time_ns.Span.millisecond in
-        Int63.to_int_exn
-          Time_ns.Span.(
-            div
-              (span + of_int63_ns (Int63.of_int 500_000))
-              (of_int63_ns (Int63.of_int 1_000_000))))
-    in
-    assert (timeout_ms >= 0);
-    wait_internal t ~timeout_ms
-  ;;
-
-  let wait t ~timeout =
-    (* From the epoll man page:
-
-       | Specifying a timeout of -1 makes epoll_wait() wait indefinitely, while
-       | specifying a timeout equal to zero makes epoll_wait() to return immediately
-       | even if no events are available (return code equal to zero). *)
-    match timeout with
-    | `Never -> wait_internal t ~timeout_ms:(-1)
-    | `Immediately -> wait_internal t ~timeout_ms:0
-    | `After span -> wait_timeout_after t span
-  ;;
-
-  let fold_ready t ~init ~f =
-    let t = in_use_exn t in
-    let ac = ref init in
-    for i = 0 to t.num_ready_events - 1 do
-      ac := f !ac (epoll_readyfd t.ready_events i) (epoll_readyflags t.ready_events i)
-    done;
-    !ac
-  ;;
-
-  let iter_ready t ~f =
-    let t = in_use_exn t in
-    for i = 0 to t.num_ready_events - 1 do
-      f (epoll_readyfd t.ready_events i) (epoll_readyflags t.ready_events i)
-    done
-  ;;
-
-  module Expert = struct
-    let clear_ready t =
-      let t = in_use_exn t in
-      t.num_ready_events <- 0
-    ;;
-  end
-
-  (* external epoll_pwait
-   *   : File_descr.t -> Events_buffer.raw -> int -> int list -> int
-   *   = "core_linux_epoll_pwait"
-   *
-   * let pwait t ~timeout sigs =
-   *   let millis = Float.iround_exn ~dir:`Zero ( Span.to_ms timeout ) in
-   *   let num_ready = epoll_pwait t.epollfd t.events millis sigs in
-   *   if num_ready = 0 then `Timeout
-   *   else `Ok { Ready_fds.num_ready ; events = t.events }
-   * ;; *)
-
-  let create = Ok create
-end
+module Epoll = Epoll.Impl
 
 let cores = Ok cores
 let isolated_cpus = Ok isolated_cpus
@@ -1335,26 +1046,32 @@ module Extended_file_attributes = struct
     [@@deriving sexp_of]
   end
 
-  external getxattr : string -> string -> Get_attr_result.t = "core_linux_getxattr"
+  external getxattr
+    :  bool
+    -> string
+    -> string
+    -> Get_attr_result.t
+    = "core_linux_getxattr"
 
   external setxattr
-    :  string
+    :  bool
+    -> string
     -> string
     -> string
     -> Int63.t
     -> Set_attr_result.t
     = "core_linux_setxattr"
 
-  let getxattr ~path ~name = getxattr path name
+  let getxattr ~follow_symlinks ~path ~name = getxattr follow_symlinks path name
 
-  let setxattr ?(how = `Set) ~path ~name ~value () =
+  let setxattr ?(how = `Set) ~follow_symlinks ~path ~name ~value () =
     let flags =
       match how with
       | `Set -> Flags.set
       | `Create -> Flags.only_create ()
       | `Replace -> Flags.only_replace ()
     in
-    setxattr path name value flags
+    setxattr follow_symlinks path name value flags
   ;;
 
   let getxattr = Ok getxattr
