@@ -12,8 +12,7 @@ module type S = sig
       ; load1 : int (** Load average over the last minute *)
       ; load5 : int (** Load average over the last 5 minutes*)
       ; load15 : int (** Load average over the last 15 minutes *)
-      ;
-        total_ram : int (** Total usable main memory *)
+      ; total_ram : int (** Total usable main memory *)
       ; free_ram : int (** Available memory size *)
       ; shared_ram : int (** Amount of shared memory *)
       ; buffer_ram : int (** Memory used by buffers *)
@@ -47,11 +46,11 @@ module type S = sig
   val sendfile
     : (?pos:int (** Defaults to 0. *)
        -> ?len:int
-       (** Defaults to length of data (file) associated with descriptor [fd]. *)
+            (** Defaults to length of data (file) associated with descriptor [fd]. *)
        -> fd:File_descr.t
        -> File_descr.t
        -> int)
-        Or_error.t
+      Or_error.t
 
   (** Type for status of SO_BINDTODEVICE socket option. The socket may either restrict the
       traffic to a given (by name, e.g. "eth0") interface, or do no restriction at all. *)
@@ -66,7 +65,7 @@ module type S = sig
 
   type tcp_bool_option =
     | TCP_CORK
-    (** (Since Linux 2.2) If set, don’t send out partial frames.  All queued partial
+        (** (Since Linux 2.2) If set, don’t send out partial frames.  All queued partial
         frames are sent when the option is cleared again.  This is useful for prepending
         headers before calling [sendfile(2)], or for throughput optimization.  As
         currently implemented, there is a 200ms ceiling on the time for which output is
@@ -75,7 +74,7 @@ module type S = sig
 
         This option should not be used in code intended to be portable. *)
     | TCP_QUICKACK
-    (** (Since Linux 2.4.4) Quick ack solves an unfortunate interaction between the
+        (** (Since Linux 2.4.4) Quick ack solves an unfortunate interaction between the
         delayed acks and the Nagle algorithm (TCP_NODELAY).  On fast LANs, the Linux TCP
         stack quickly reaches a CWND (congestion window) of 1 (Linux interprets this as "1
         unacknowledged packet", BSD/Windows and others consider it "1 unacknowledged
@@ -91,7 +90,7 @@ module type S = sig
 
   type tcp_string_option =
     | TCP_CONGESTION
-    (** (Since Linux 2.6.13) Get or set the congestion-control algorithm for this socket.
+        (** (Since Linux 2.6.13) Get or set the congestion-control algorithm for this socket.
 
         The algorithm "reno" is always permitted; other algorithms may be available,
         depending on kernel configuration and loaded modules (see
@@ -132,7 +131,7 @@ module type S = sig
        -> ?len:int (** default = [Bytes.length buf - pos] *)
        -> Bytes.t
        -> int option)
-        Or_error.t
+      Or_error.t
 
   (** [send_no_sigpipe sock ?pos ?len buf] tries to do a blocking send on socket [sock]
       given buffer [buf], offset [pos] and length [len]. Prevents [SIGPIPE], i.e., raises
@@ -146,7 +145,7 @@ module type S = sig
        -> ?len:int (** default = [Bytes.length buf - pos] *)
        -> Bytes.t
        -> int)
-        Or_error.t
+      Or_error.t
 
   (** [sendmsg_nonblocking_no_sigpipe sock ?count iovecs] tries to do a nonblocking send
       on socket [sock] using [count] I/O-vectors [iovecs].  Prevents [SIGPIPE],
@@ -325,6 +324,54 @@ module type S = sig
     end
   end
 
+  (** {2 Memfd functions} *)
+
+  module Memfd : sig
+    module Flags : sig
+      type t [@@deriving sexp_of]
+
+      include Flags.S with type t := t
+
+      (** MFD_CLOEXEC *)
+      val cloexec : t
+
+      (** MFD_ALLOW_SEALING *)
+      val allow_sealing : t
+
+      (** MFD_HUGETLB *)
+      val hugetlb : t
+
+      (** MFD_NOEXEC_SEAL *)
+      val noexec_seal : t
+
+      (** MFD_EXEC *)
+      val exec : t
+
+      (** MFD_HUGE_2MB *)
+      val huge_2mb : t
+
+      (** MFD_HUGE_1GB *)
+      val huge_1gb : t
+    end
+
+    type t = private File_descr.t [@@deriving sexp_of]
+
+    val to_file_descr : t -> File_descr.t
+
+    (** From memfd_create():
+
+        [create] creates an anonymous file and returns a file descriptor that refers to
+        it.  The file behaves like a regular file, and so can be modified, truncated,
+        memory-mapped, and so on.  However, unlike a regular file, it lives in RAM and has
+        a volatile backing storage.
+
+        Once all references to the file are dropped, it is automatically released.
+        Anonymous memory is used for all backing pages of the file.  Therefore, files
+        created by [create] have the same semantics as other anonymous memory allocations
+        such as those allocated using [mmap] with the [MAP_ANONYMOUS] flag. *)
+    val create : (?flags:Flags.t -> ?initial_size:int -> string -> t) Or_error.t
+  end
+
   (** {2 Parent death notifications} *)
 
   (** [pr_set_pdeathsig s] sets the signal [s] to be sent to the executing process when
@@ -452,7 +499,6 @@ module type S = sig
       that the calling thread's priority will be retrieved. *)
   val getpriority : (?pid:Pid.t -> unit -> Priority.t) Or_error.t
 
-
   (** [get_ipv4_address_for_interface "eth0"] returns the IP address assigned to eth0, or
       throws an exception if no IP address is configured. *)
   val get_ipv4_address_for_interface : (string -> string) Or_error.t
@@ -479,131 +525,7 @@ module type S = sig
 
   val get_bind_to_interface : (File_descr.t -> Bound_to_interface.t) Or_error.t
 
-  (** epoll(): a Linux I/O multiplexer of the same family as select() or poll().  Its main
-      differences are support for Edge- or Level-triggered notifications (we're using
-      Level-triggered to emulate "select") and much better scaling with the number of file
-      descriptors.
-
-      See the man pages for a full description of the epoll facility. *)
-
-  module Epoll : sig
-    module Flags : sig
-      (** An [Epoll.Flags.t] is an immutable set of flags for which one can register
-          interest in a file descriptor.  It is implemented as a bitmask, and so all
-          operations (+, -, etc.) are constant time with no allocation.
-
-          [sexp_of_t] produces a human-readable list of bits, e.g., "(in out)". *)
-      type t [@@deriving sexp_of]
-
-      include Flags.S with type t := t
-
-      (** The names of the flags match the man pages.  E.g. [in_] = "EPOLLIN", [out] =
-          "EPOLLOUT", etc. *)
-
-      (** Associated fd is readable                      *)
-      val none : t
-
-      (** Associated fd is readable                      *)
-      val in_ : t
-
-      (** Associated fd is writable                      *)
-      val out : t
-
-      (*_ val rdhup   : t (\* Event flag For detecting tcp half-close        *\) *)
-
-      (** Urgent data available                          *)
-      val pri : t
-
-      (** Error condition (always on, no need to set it) *)
-      val err : t
-
-      (** Hang up happened (always on)                   *)
-      val hup : t
-
-      (** Edge-Triggered behavior (see man page)         *)
-      val et : t
-
-      (** One-shot behavior for the associated fd        *)
-      val oneshot : t
-    end
-
-    (** An [Epoll.t] maintains a map from [File_descr.t] to [Flags.t], where the domain is
-        the set of file descriptors that one is interested in, and the flags associated
-        with each file descriptor specify the types of events one is interested in being
-        notified about for that file descriptor.  Our implementation maintains a
-        user-level table equivalent to the kernel epoll set, so that [sexp_of_t] produces
-        useful human-readable information, and so that we can present our standard table
-        interface.
-
-        The implementation assumes that one never closes a file descriptor that is the
-        domain of an [Epoll.t], since doing so might remove the [fd] from the kernel epoll
-        set without the implementation's knowledge.
-
-        An [Epoll.t] also has a buffer that is used to store the set of ready [fd]s
-        returned by calling [wait]. *)
-    type t [@@deriving sexp_of]
-
-    val invariant : t -> unit
-
-    (** [create ~num_file_descrs] creates a new epoll set able to watch file descriptors
-        in \[0, [num_file_descrs]).  Additionally, the set allocates space for reading the
-        "ready" events when [wait] returns, allowing for up to [max_ready_events] to be
-        returned in a single call to [wait]. *)
-    val create : (num_file_descrs:int -> max_ready_events:int -> t) Or_error.t
-
-    val close : t -> unit
-
-    (** Map operations *)
-
-    (** [find] raises in the case that [t] is closed. *)
-    val find : t -> File_descr.t -> Flags.t option
-
-    val find_exn : t -> File_descr.t -> Flags.t
-    val set : t -> File_descr.t -> Flags.t -> unit
-    val remove : t -> File_descr.t -> unit
-    val iter : t -> f:(File_descr.t -> Flags.t -> unit) -> unit
-    val fold : t -> init:'a -> f:(File_descr.t -> Flags.t -> 'a -> 'a) -> 'a
-
-
-    (** [wait t ~timeout] blocks until at least one file descriptor in [t] is ready for
-        one of the events it is being watched for, or [timeout] passes.  [wait] side
-        effects [t] by storing the ready set in it.  One can subsequently access the ready
-        set by calling [iter_ready] or [fold_ready].
-
-        With [wait ~timeout:(`After span)], [span <= 0] is treated as [0].  If [span > 0],
-        then [span] is rounded to the nearest millisecond, with a minimum value of one
-        millisecond.
-
-        Note that this method should not be considered thread-safe.  There is mutable
-        state in [t] that will be changed by invocations to [wait] that cannot be
-        prevented by mutexes around [wait]. *)
-    val wait
-      :  t
-      -> timeout:[ `Never | `Immediately | `After of Time_ns.Span.t ]
-      -> [ `Ok | `Timeout ]
-
-    (** [wait_timeout_after t span = wait t ~timeout:(`After span)].  [wait_timeout_after]
-        is a performance hack to avoid allocating [`After span]. *)
-    val wait_timeout_after : t -> Time_ns.Span.t -> [ `Ok | `Timeout ]
-
-    (** [iter_ready] and [fold_ready] iterate over the ready set computed by the last
-        call to [wait]. *)
-    val iter_ready : t -> f:(File_descr.t -> Flags.t -> unit) -> unit
-
-    val fold_ready : t -> init:'a -> f:('a -> File_descr.t -> Flags.t -> 'a) -> 'a
-
-    module Expert : sig
-      (** [clear_ready t] sets the number of ready events in [t] to [0]. This
-          should be called after all the events in [t] have been processed,
-          following a call to {!wait}. *)
-      val clear_ready : t -> unit
-    end
-
-    (*_
-      (* pwait -> with the specified sigmask, analogous to pselect *)
-      (* val pwait   : t -> timeout:Span.t -> int list -> [ `Ok of Ready_fds.t | `Timeout ] *)
-    *)
-  end
+  module Epoll : Epoll.S
 
   module Extended_file_attributes : sig
     (** Extended attributes are name:value pairs associated with inodes (files,
@@ -638,7 +560,9 @@ module type S = sig
       [@@deriving sexp_of]
     end
 
-    val getxattr : (path:string -> name:string -> Get_attr_result.t) Or_error.t
+    val getxattr
+      : (follow_symlinks:bool -> path:string -> name:string -> Get_attr_result.t)
+        Or_error.t
 
     (** [setxattr] sets the value of the extended attribute identified by name and
         associated with the given path in the filesystem.
@@ -664,11 +588,12 @@ module type S = sig
 
     val setxattr
       : (?how:[ `Set | `Create | `Replace ]
+         -> follow_symlinks:bool
          -> path:string
          -> name:string
          -> value:string
          -> unit
          -> Set_attr_result.t)
-          Or_error.t
+        Or_error.t
   end
 end
