@@ -48,7 +48,8 @@ let%test_module _ =
         (1999-12-31 19:00:07.000000-05:00)
         (1999-12-31 19:00:08.000000-05:00)
         (1999-12-31 19:00:09.000000-05:00)
-        (1999-12-31 19:00:10.000000-05:00) |}];
+        (1999-12-31 19:00:10.000000-05:00)
+        |}];
       time := !time +. one_hour;
       tick_tock 15;
       [%expect
@@ -67,7 +68,8 @@ let%test_module _ =
         (1999-12-31 21:25:49.789822-05:00)
         (1999-12-31 21:26:02.359724-05:00)
         (1999-12-31 21:26:10.298530-05:00)
-        (1999-12-31 21:26:15.428435-05:00) |}];
+        (1999-12-31 21:26:15.428435-05:00)
+        |}];
       time := !time -. 30.;
       (* moving time backwards does not violate monotonicity *)
       tick_tock 10;
@@ -82,7 +84,8 @@ let%test_module _ =
         (1999-12-31 21:26:29.996873-05:00)
         (1999-12-31 21:26:30.927624-05:00)
         (1999-12-31 21:26:31.806929-05:00)
-        (1999-12-31 21:26:32.655030-05:00) |}];
+        (1999-12-31 21:26:32.655030-05:00)
+        |}];
       time := !time -. half_a_year;
       (* even if we attempt to move time backwards a lot *)
       tick_tock 10;
@@ -97,7 +100,8 @@ let%test_module _ =
         (1999-12-31 21:26:38.354661-05:00)
         (1999-12-31 21:26:39.156113-05:00)
         (1999-12-31 21:26:39.956994-05:00)
-        (1999-12-31 21:26:40.757529-05:00) |}]
+        (1999-12-31 21:26:40.757529-05:00)
+        |}]
     ;;
 
     (* monotonicity testing *)
@@ -280,7 +284,7 @@ let%test_module _ =
     let%expect_test "Test allocation" =
       ignore
         (Expect_test_helpers_core.require_no_allocation [%here] (fun () -> now ()) : t);
-      [%expect {|  |}]
+      [%expect {| |}]
     ;;
 
     [%%endif]
